/*****************************************************
Project : usb-lpt
Version : 1.0
Author  :  ..
Chip type           : ATmega8515
Program type        : Application
Clock frequency     : 7,372800 MHz
Memory model        : Small
External SRAM size  : 0
Data Stack size     : 128
*****************************************************/

#include <mega8515.h>
#include <mega8515bit.h>
#include <bin.h>

// Standard Input/Output functions
#include <stdio.h>

// Declare your global variables here
#pragma warn-
eeprom unsigned char bufdat[256];
#pragma warn+
unsigned char mod, adr, dat; 
unsigned char RegDR, RegSR, RegCR;
flash unsigned char name[]={" COM-LPT V1.0 "};

#define MASKMOD 0x80
#define LED PORTD.5   // 
#define ON 0          // 
#define OFF 1         // 

#define PORT_CR PORTC
#define PORT_SR PINB
#define PORT_DRCFG DDRA
#define PORT_DRINP PINA
#define PORT_DROUT PORTA

#define M_RD b00000000
#define M_WR b00000001
#define M_LPT b00000000
#define M_EEPROM b00000010

#define MaskSR b10000000
#define MaskCR b00001011
#define AdrDR 0
#define AdrSR 1
#define AdrCR 2
#define MaxAdrEEPROM b01111111

//   UART
void UART_TXD( unsigned char data )
{
    /*     */
    while ( !( UCSRA & (1<<UDRE)) )
      #asm("wdr"); //   
    /*        */
    UDR = data;
}

//   UART
unsigned char UART_RXD( void )
{
    /*       */
    while ( !(UCSRA & (1<<RXC)) )
      #asm("wdr"); //   
    /*        */
    return UDR;
}

//     LPT
void WR_LPT(unsigned char adr, unsigned char dat)
{
    if(adr==AdrDR) {RegDR=dat; PORT_DRCFG=0xFF; PORT_DROUT=dat;}
    if(adr==AdrSR) {RegSR=dat;}
    if(adr==AdrCR) {RegCR=dat; PORT_CR=dat ^ MaskCR;}
}

//    LPT
unsigned char RD_LPT(unsigned char adr)
{
    unsigned char d=0;
    if(adr==AdrDR) {PORT_DRCFG=0; PORT_DROUT=0xFF; d=PORT_DRINP;}
    if(adr==AdrSR) d=PORT_SR ^ MaskSR;
    if(adr==AdrCR) d=PORT_CR ^ MaskCR;
    return d;
}

//     EEPROM
void WR_EEPROM(unsigned char adr, unsigned char dat)
{
    bufdat[adr & MaxAdrEEPROM]=dat;
}

//    EEPROM
unsigned char RD_EEPROM(unsigned char adr)
{
    unsigned char d;
    d=bufdat[adr & MaxAdrEEPROM];
    return d;
}

void main(void)
{
// Declare your local variables here

// Input/Output Ports initialization
// Port A initialization
PORTA=0xFF;
DDRA=0x00;

// Port B initialization
PORTB=0xFF;
DDRB=0x00;

// Port C initialization
PORTC=0xFF;
DDRC=0xFF;

// Port D initialization
// Func7=In Func6=In Func5=Out Func4=In Func3=In Func2=In Func1=Out Func0=In 
// State7=T State6=T State5=0 State4=T State3=T State2=T State1=0 State0=T 
PORTD=0x00;
DDRD=0x22;

// Port E initialization
// Func2=In Func1=In Func0=In 
// State2=T State1=T State0=T 
PORTE=0x00;
DDRE=0x00;

// Timer/Counter 0 initialization
// Clock source: System Clock
// Clock value: 7,200 kHz
// Mode: Normal top=FFh
// OC0 output: Disconnected
TCCR0=0x05;
TCNT0=0x00;
OCR0=0x00;

// Timer/Counter 1 initialization
// Clock source: System Clock
// Clock value: 115,200 kHz
// Mode: Normal top=FFFFh
// OC1A output: Discon.
// OC1B output: Discon.
// Noise Canceler: Off
// Input Capture on Falling Edge
// Timer 1 Overflow Interrupt: Off
// Input Capture Interrupt: Off
// Compare A Match Interrupt: Off
// Compare B Match Interrupt: Off
TCCR1A=0x00;
TCCR1B=0x03;
TCNT1H=0x00;
TCNT1L=0x00;
ICR1H=0x00;
ICR1L=0x00;
OCR1AH=0x00;
OCR1AL=0x00;
OCR1BH=0x00;
OCR1BL=0x00;

// External Interrupt(s) initialization
// INT0: Off
// INT1: Off
// INT2: Off
MCUCR=0x00;
EMCUCR=0x00;

// Timer(s)/Counter(s) Interrupt(s) initialization
TIMSK=0x00;

// USART initialization
// Communication Parameters: 8 Data, 1 Stop, No Parity
// USART Receiver: On
// USART Transmitter: On
// USART Mode: Asynchronous
// USART Baud rate: 115200
UCSRA=0x00;
UCSRB=0x18;
UCSRC=0x86;
UBRRH=0x00;
UBRRL=0x03;

// Analog Comparator initialization
// Analog Comparator: Off
// Analog Comparator Input Capture by Timer/Counter 1: Off
ACSR=0x80;

// Watchdog Timer initialization
// Watchdog Timer Prescaler: OSC/2048k
#pragma optsize-
WDTCR=0x1F;
WDTCR=0x0F;
#ifdef _OPTIMIZE_SIZE_
#pragma optsize+
#endif

while (1)
      {
      #asm("WDR"); //   
      LED=ON; //  
      mod=UART_RXD(); //    UART
      LED=OFF; //  
      adr=UART_RXD(); //    UART
      switch (mod) //  
        {
         case M_RD | M_LPT:
           dat=RD_LPT(adr);
           UART_TXD(dat);
           break;
         case M_RD | M_EEPROM:
           dat=RD_EEPROM(adr);
           UART_TXD(dat);
           break;
         case M_WR | M_LPT:
           dat=UART_RXD();
           WR_LPT(adr, dat);
           break;
         case M_WR | M_EEPROM:
           dat=UART_RXD();
           WR_EEPROM(adr, dat);
           break;
         default:
        };
      };
}
